<?php

namespace Drupal\Tests\views_exposed_input_token\Unit;

use Drupal\Core\EventSubscriber\AjaxResponseSubscriber;
use Drupal\Core\EventSubscriber\MainContentViewSubscriber;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Tests\UnitTestCase;
use Drupal\views\ViewExecutable;

/**
 * Unit tests for the `_view_exposed_input_token_get_query_string()` function.
 *
 * Copyright (C) 2024  Library Solutions, LLC (et al.).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * @group views_exposed_input_token
 */
class ViewsExposedInputTokenTest extends UnitTestCase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'views_exposed_input_token',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    require_once __DIR__ . '/../../../views_exposed_input_token.module';
    parent::setUp();
  }

  /**
   * A data provider with a list of inputs and expected URL query strings.
   */
  public function exposedInputProvider() {
    return [
      [
        [],
        '',
      ],
      [
        [
          'field' => 'value',
        ],
        '?field=value',
      ],
      [
        [
          'field' => 'value',
          'foo' => 'bar',
        ],
        '?field=value&foo=bar',
      ],
    ];
  }

  /**
   * Tests that a URL query string is generated.
   *
   * @dataProvider exposedInputProvider
   */
  public function testGetQueryString($input, $expected) {
    $view = $this->prophesize(ViewExecutable::class);
    $view->getExposedInput()->willReturn($input);
    $view->getCurrentPage()->willReturn(0);

    $this->assertEquals($expected, \_views_exposed_input_token_get_query_string($view->reveal()));
  }

  /**
   * A data provider with a list of forbidden keys.
   */
  public function forbiddenKeysProvider() {
    return [
      ['view_name'],
      ['view_display_id'],
      ['view_args'],
      ['view_path'],
      ['view_dom_id'],
      ['pager_element'],
      ['view_base_path'],
      [AjaxResponseSubscriber::AJAX_REQUEST_PARAMETER],
      [FormBuilderInterface::AJAX_FORM_REQUEST],
      [MainContentViewSubscriber::WRAPPER_FORMAT],
    ];
  }

  /**
   * Test that forbidden keys are not included in the URL query string.
   *
   * @dataProvider forbiddenKeysProvider
   */
  public function testForbiddenKeysAreRemoved($forbidden_key) {
    $view = $this->prophesize(ViewExecutable::class);
    $view->getExposedInput()->willReturn([$forbidden_key => 'test']);
    $view->getCurrentPage()->willReturn(0);

    $this->assertStringNotContainsString($forbidden_key, \_views_exposed_input_token_get_query_string($view->reveal()));
  }

  /**
   * A data provider with pager values.
   */
  public function pagerDataProvider() {
    return [
      [0, '?field=value'],
      [1, '?field=value&page=1'],
    ];
  }

  /**
   * Tests the pager portion of the URL query string.
   *
   * @dataProvider pagerDataProvider
   */
  public function testPager($pager, $expected) {
    $view = $this->prophesize(ViewExecutable::class);
    $view->getExposedInput()->willReturn(['field' => 'value']);
    $view->getCurrentPage()->willReturn($pager);

    $this->assertEquals($expected, \_views_exposed_input_token_get_query_string($view->reveal()));
  }

}
